package com.massa.mrules.demos.insurance;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.massa.log.Log;
import com.massa.log.LogFactory;
import com.massa.mrules.context.compile.CompilationLevel;
import com.massa.mrules.demos.insurance.model.Price;
import com.massa.mrules.exception.IMRulesException;
import com.massa.mrules.extensions.dsl.factory.NativeLanguageFactory;
import com.massa.mrules.set.IMRuleExecutionSet;
import com.massa.util.classloader.Resource;
import com.massa.util.classloader.ResourceLoader;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameters;

import java.util.Collection;
import java.util.stream.Collectors;

import lombok.SneakyThrows;

/**
 * Finds dynamicaly test data files and runs them all.
 */
@RunWith(Parameterized.class)
public class DemoInsuranceTest {
	private static final Log LOG = LogFactory.getLog(DemoInsuranceTest.class.getName());
	
	private static ObjectMapper MAPPER;
	private static RulesetService service;
	
	private String testFileName;

	public DemoInsuranceTest(String testFileName) {
		super();
		this.testFileName = testFileName;
	}

	@Parameters(name="jdd: {0}")
	@SneakyThrows
	public static Collection<Object[]> configs() {
		final Collection<Resource> files = ResourceLoader.getMatchingResources("datasets", ".*\\.json");
		return files.stream().map(e -> new Object[]{e.getFullResourceName()}).collect(Collectors.toList());
	}
	
	@BeforeClass
	@SneakyThrows
	public static void setupClass() {
		MAPPER = new ObjectMapper();
		MAPPER.registerModule(new JavaTimeModule());
		IMRuleExecutionSet ruleSet = new NativeLanguageFactory(CompilationLevel.STANDARD).read(ResourceLoader.getResourceAsStream("ruleset.mrl"));
		service = new RulesetService(ruleSet);
	}
	
	@AfterClass
	public static void tearDownClass() {
		MAPPER = null;
		service = null;
	}
	
	@Test
	@SneakyThrows
	public void demoInsurance() {
		if (LOG.isInfoEnabled()) LOG.info("Starting test for Dataset " + testFileName);
		
		if (LOG.isDebugEnabled()) LOG.debug("Reading Dataset");
		DemoInsuranceDataset dataset = MAPPER.readerFor(DemoInsuranceDataset.class).readValue(ResourceLoader.getResourceAsStream(testFileName));
		
		if (LOG.isDebugEnabled()) LOG.debug("Running rule engine");
		Price price = null;
		IMRulesException raisedException = null;
		try {
			price = service.execute(dataset.getParameters(), dataset.getCustomer(), dataset.getContract());
		} catch (Exception e) {
			if (! IMRulesException.class.isAssignableFrom(e.getClass())) {
				throw e;
			}
			raisedException = (IMRulesException) e;
		}

		if (LOG.isDebugEnabled()) LOG.debug("Verifying results");
		if (raisedException == null && dataset.getExpectedException() != null) {
			Assert.fail("An exception should be raised.");
		} else if (raisedException != null && dataset.getExpectedException() == null) {
			Assert.fail("An unexpected exception has been raised: " + raisedException.getMessage());
		} else if (raisedException != null && dataset.getExpectedException() != null) {
			if (LOG.isDebugEnabled()) LOG.debug("Verifying raised exception");
			Assert.assertEquals(dataset.getExpectedException().getType(), raisedException.getClass());
			Assert.assertEquals(dataset.getExpectedException().getCode(), raisedException.getInfoBlock().getMessageInfo().getMessageCode().getErrorCode());
			Assert.assertEquals(dataset.getExpectedException().getMessage(), raisedException.getMessage());
		} else {
			if (LOG.isDebugEnabled()) LOG.debug("Verifying price");
			Assert.assertEquals(dataset.getExpectedPrice(), price);
		}

		if (LOG.isInfoEnabled()) LOG.info("Ending test for Dataset " + testFileName);
	}
}
