package com.massa.mrules.demos.insurance;

import com.massa.mrules.addon.DefaultVariablesManager;
import com.massa.mrules.addon.DefaultVariablesManager.AbstractVariableAccessor;
import com.massa.mrules.addon.IAddonsManager;
import com.massa.mrules.addon.MAddonsFinder;
import com.massa.mrules.context.IContext;
import com.massa.mrules.context.execute.IExecutionContext;
import com.massa.mrules.demos.insurance.model.Parameters;
import com.massa.mrules.exception.MAccessorException;
import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.exception.MRuleInternalException;
import com.massa.util.ConfigDiscovery;

/**
 * Declares the global variable #PARAMETERS. cf. mrules-addons.xml.
 */
public class InsuranceVariablesManager implements IAddonsManager {

	/**
	 * Manually registers addons using {@link ConfigDiscovery}.
	 * Variable Accessors implementation ID <b>must</b> start with the "#" character.
	 */
	@Override
	public void registerAddons() throws MConfigurationException, MRuleInternalException {
		MAddonsFinder.registerAddon(new InsuranceVariablesManager.ParametersGlobalVariable());
	}

	/**
	 * "#PARAMETERS" global variable accessor.
	 */
	public static class ParametersGlobalVariable extends AbstractVariableAccessor {

		public static final String IMPLEMENTATION_ID = DefaultVariablesManager.GLOBAL_VAR_PREFIX + "PARAMETERS";
		private static final long serialVersionUID = 1L;

		@Override
		public Class<?> getDefaultType(final IContext ctxt) {
			return Parameters.class;
		}

		@Override
		public String getImplementationId() {
			return ParametersGlobalVariable.IMPLEMENTATION_ID;
		}

		@Override
		public Object getVarValue(final IExecutionContext ctxt) throws MAccessorException {
			//Here, contact database to retrieve parameter values.
			return null;
		}

		@Override
		public boolean isCacheUsed() {
			return true;
		}

		@Override
		public boolean isShouldBeWrappedToConstantValue() {
			return false;
		}
	}
}
