package com.massa.mrules.demos.insurance;

import com.massa.mrules.addon.IAddonsManager;
import com.massa.mrules.addon.MAddonsFinder;
import com.massa.mrules.context.execute.IExecutionContext;
import com.massa.mrules.demos.insurance.InsuranceVariablesManager.ParametersGlobalVariable;
import com.massa.mrules.demos.insurance.model.Parameters;
import com.massa.mrules.exception.MAccessorException;
import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.exception.MRuleInternalException;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;

public class MockInsuranceVariablesManager implements IAddonsManager {

	/**
	 * Unregisters real implentation of PARAMETER variable and registers mock.
	 */
	@Override
	public void registerAddons() throws MConfigurationException, MRuleInternalException {
		MAddonsFinder.unregisterAddon(InsuranceVariablesManager.ParametersGlobalVariable.IMPLEMENTATION_ID);
		MAddonsFinder.registerAddon(new MockInsuranceVariablesManager.MockParametersGlobalVariable());
	}

	/**
	 * Mocked "#PARAMETERS" system variable accessor.
	 */
	public static class MockParametersGlobalVariable extends ParametersGlobalVariable {

		private static final long serialVersionUID = 1L;
		@Override
		public Object getVarValue(final IExecutionContext ctxt) throws MAccessorException {
			//Here, we mock the parameters.
			return new MockParameters(18, 70);
		}
	}

	@Getter
	@Setter
	@AllArgsConstructor
	public static final class MockParameters implements Parameters {
		private int majorityAge;
		private int elderlyAge;
	}
}
