package com.massa.mrules;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;

import org.springframework.batch.core.configuration.annotation.EnableBatchProcessing;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.context.annotation.ComponentScan;

import com.massa.mrules.extensions.dsl.editor.MRulesDslEditorApplication;
import com.massa.util.classloader.ResourceLoader;

import lombok.SneakyThrows;

/**
 * Launches the application.
 */
@EnableBatchProcessing
@ComponentScan(basePackages={"com.massa.mrules.demos.insurance"})
@EnableAutoConfiguration
@SpringBootApplication
public class BatchApplication extends MRulesDslEditorApplication {

	public static final String RULESET_CONFIG_FILE = "./tmp/ruleset.mrl";

	@SneakyThrows
	public static void main(String[] args) {
		// First, for the demo, creating tmp directory with ruleset initial configuration
		// In a real case, the config should be stored in database for example.
		new File ("./tmp/").mkdirs();
		try (
				InputStream in = ResourceLoader.getResourceAsStream("ruleset.mrl");
				OutputStream out = new BufferedOutputStream(new FileOutputStream(RULESET_CONFIG_FILE));
		) {
			int i = in.read();
			while (i >= 0) {
				out.write(i);
				i = in.read();
			}
		}

		//Then run the app
		new SpringApplicationBuilder(BatchApplication.class).run(args);
	}
	

}
