package com.massa.mrules.demos.insurance.batch.components;

import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;

import com.massa.mrules.context.execute.MExecutionContext;
import com.massa.mrules.demos.insurance.model.Price;
import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.exception.MExecutionException;
import com.massa.mrules.executable.IExecutionResult;
import com.massa.mrules.set.IMRuleExecutionSet;

/**
 * Receives data read by the reader, processes it via MRules and returns the result.
 */
public class DemoInsuranceProcessor implements ItemProcessor<DemoInsuranceDataset, DemoInsuranceResult> {
	
	@Autowired
	@Qualifier("batchRuleSet")
	private IMRuleExecutionSet ruleSet;

	@Override
	public DemoInsuranceResult process(DemoInsuranceDataset item) throws MConfigurationException {
		RulesetInputBean data = new RulesetInputBean();
		data.setParameters(item.getParameters());
		data.setCustomer(item.getCustomer());
		data.setContract(item.getContract());

		@SuppressWarnings("unchecked")
		MExecutionContext<RulesetInputBean> ctxt = (MExecutionContext<RulesetInputBean>) ruleSet.getContextFactory().getExecutionContext();
		ctxt.setInput(data);
		
		DemoInsuranceResult result = new DemoInsuranceResult();

		try {
			IExecutionResult ruleSetExecutionResult = ctxt.execute();
			result.setPrice((Price) ruleSetExecutionResult.getReturnedValue());
		} catch (MExecutionException e) {
			result.setError(e.getMessage());
		}

		return result;
	}
}