package com.massa.mrules.demos.insurance.batch.components;

import java.util.Iterator;

import org.springframework.batch.item.ItemReader;
import org.springframework.batch.item.NonTransientResourceException;
import org.springframework.batch.item.ParseException;
import org.springframework.batch.item.UnexpectedInputException;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.massa.util.classloader.Resource;
import com.massa.util.classloader.ResourceLoader;

import lombok.SneakyThrows;

/**
 * Reads all datasets from resources to send them to the processor at each run.
 */
public class DemoInsuranceReader implements ItemReader<DemoInsuranceDataset> {
	
	private static ObjectMapper MAPPER = new ObjectMapper();
	static {
		MAPPER.registerModule(new JavaTimeModule());
	}

	private Iterator<Resource> items;
	
	@SneakyThrows
	public DemoInsuranceReader() {
		items = ResourceLoader.getMatchingResources("datasets", ".*\\.json").iterator();
	}
	
	@Override
	public DemoInsuranceDataset read() throws Exception, UnexpectedInputException, ParseException, NonTransientResourceException {
		if (items.hasNext()) {
			Resource next = items.next();
			DemoInsuranceDataset dataset = MAPPER.readerFor(DemoInsuranceDataset.class).readValue(ResourceLoader.getResourceAsStream(next.getFullResourceName()));
			return dataset;
		}
		return null;
	}
}