package com.massa.mrules.demos.insurance.batch.config;

import org.springframework.batch.core.Job;
import org.springframework.batch.core.Step;
import org.springframework.batch.core.configuration.annotation.JobBuilderFactory;
import org.springframework.batch.core.configuration.annotation.StepBuilderFactory;
import org.springframework.batch.core.job.builder.FlowBuilder;
import org.springframework.batch.core.job.builder.FlowJobBuilder;
import org.springframework.batch.core.job.builder.JobBuilder;
import org.springframework.batch.core.job.flow.support.SimpleFlow;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.batch.item.ItemReader;
import org.springframework.batch.item.ItemWriter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;
import org.springframework.core.task.SimpleAsyncTaskExecutor;

import com.massa.mrules.demos.insurance.batch.components.DemoInsuranceDataset;
import com.massa.mrules.demos.insurance.batch.components.DemoInsuranceProcessor;
import com.massa.mrules.demos.insurance.batch.components.DemoInsuranceReader;
import com.massa.mrules.demos.insurance.batch.components.DemoInsuranceResult;
import com.massa.mrules.demos.insurance.batch.components.DemoInsuranceWriter;

/**
 * Configures beans that will be included in Spring batch Job.<br>
 * Please note that these beans are not Singleton but prototypes, as new instances are needed for every job runs.
 */
@Configuration
public class BatchConfiguration {
	
	@Autowired
	private JobBuilderFactory jobBuilderFactory;

	@Autowired
	private StepBuilderFactory stepBuilderFactory;

	@Bean("mrulesReader")
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public ItemReader<DemoInsuranceDataset> mrulesReader() {
        return new DemoInsuranceReader();
	}

	@Bean("mrulesProcessor")
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public ItemProcessor<DemoInsuranceDataset, DemoInsuranceResult> mrulesProcessor() {
        return new DemoInsuranceProcessor();
	}

	@Bean("mrulesWriter")
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public ItemWriter<DemoInsuranceResult> mrulesWriter() {
        return new DemoInsuranceWriter();
	}

	@Bean("mrulesJob")
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	@Autowired
	public Job mrulesJob(@Qualifier("mrulesStep") Step mrulesStep) {
		JobBuilder jb = jobBuilderFactory.get("mrulesJob");
		
		jb.preventRestart().incrementer(new RunIdIncrementer());
		
		SimpleFlow flow = new FlowBuilder<SimpleFlow>("SplitFlow")
				.split(new SimpleAsyncTaskExecutor())
				.add(new FlowBuilder<SimpleFlow>("mrulesFlow").start(mrulesStep).build()).build();
		
		SimpleFlow splitFlow = new FlowBuilder<SimpleFlow>("MainFlow").start(flow).end();
		FlowJobBuilder jfb  = jb.start(splitFlow).end();
		
		return jfb.build();
	}

	@Bean("mrulesStep")
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	@Autowired
	public Step mrulesStep(
			@Qualifier("mrulesReader") ItemReader<DemoInsuranceDataset> reader,
			@Qualifier("mrulesProcessor") ItemProcessor<DemoInsuranceDataset, DemoInsuranceResult> processor,
			@Qualifier("mrulesWriter") ItemWriter<DemoInsuranceResult> writer
	) {
        return stepBuilderFactory.get("mrulesStep")
            .<DemoInsuranceDataset, DemoInsuranceResult> chunk(10)
            .reader(reader)
            .processor(processor)
            .writer(writer)
            .build();
	}
}
