package com.massa.mrules.demos.insurance.batch.config;

import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.extensions.dsl.builder.MrlRuleEngineConfigHolder;
import com.massa.mrules.extensions.dsl.factory.NativeLanguageFactory;
import com.massa.mrules.extensions.spring.AbstractMRulesFactoryBean.AdditionalProperty;
import com.massa.mrules.extensions.spring.MRulesFactoryBean;
import com.massa.mrules.set.IMRuleExecutionSet;

/**
 * Configures MRules related beans, i.e. the ruleset.<br>
 * Please note that the ruleset bean is not Singleton for Spring but prototypes.
 * This allows to check if configuration has changed between two runs of the batch.
 * This operation is performed automatically by the MRulesBuilder, which is used by the MRulesFactoryBean.
 */
@Configuration
public class MRulesConfiguration {

	/**
	 * Creates a rule engine, managed by Spring, via the MRules Factory Bean.
	 */
	@Bean(name = "batchRuleSet")
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public FactoryBean<IMRuleExecutionSet> getFactRuleEngine() throws MConfigurationException {
		final MRulesFactoryBean fact = new MRulesFactoryBean();

		fact.setUri("BATCH_RULESET");
		fact.setFactoryImpl(NativeLanguageFactory.class.getName());
		fact.setConfigHolderImpl(MrlRuleEngineConfigHolder.class.getName());
		fact.addProperty(new AdditionalProperty("mrlconfigholder.mrlfile", "file:///./tmp/ruleset.mrl"));

		return fact;
	}

}
