/*
 * Copyright (c) 2017 N. Billard.
 */

package com.massa.mrules.extensions.dsl.editor.web;

import java.io.File;

import org.apache.commons.io.FileUtils;
import org.vaadin.aceeditor.AceEditor;

import com.massa.mrules.BatchApplication;
import com.massa.util.property.PropertyUtils;
import com.vaadin.server.VaadinRequest;
import com.vaadin.ui.Notification;

import lombok.SneakyThrows;

/**
 * New UI, replacing the original discovered one dynamically thanks to {@link ReplaceOriginalUI}.<br>
 * We use this trick in a demo, but a production app should create its own UI.
 */
public class ExtMRulesDslEditorUI extends MRulesDslEditorUI {

	private static final long serialVersionUID = 1L;

	private AceEditor retrievedEditor;

	@Override
	@SneakyThrows
	protected void init(final VaadinRequest request) {
		super.init(request);

		//Retrieve the AceEditor instance declared in a provate field in the super class
		retrievedEditor = (AceEditor) PropertyUtils.get(this, "!editor");
		
		//Read the current ruleset configuration
		String rulesetConfig = FileUtils.readFileToString(new File(BatchApplication.RULESET_CONFIG_FILE));
		
		//Init the editor with the ruleset configuration
		retrievedEditor.setValue(rulesetConfig);
	}

	@SneakyThrows
	protected void save() {
		//Save the ruleset configuration to the file.
		String rulesetConfig = retrievedEditor.getValue();
		File file = new File(BatchApplication.RULESET_CONFIG_FILE);
		FileUtils.writeStringToFile(file, rulesetConfig);
		
		//Notify the user
		Notification.show("Saved", "Saved to " + file.getAbsolutePath(), Notification.Type.TRAY_NOTIFICATION);
	}

}
